local L = LibStub("AceLocale-3.0"):GetLocale("NeedjaTracker", true);

NeedjaTracker = LibStub("AceAddon-3.0"):NewAddon("NeedjaTracker", "AceConsole-3.0", "AceEvent-3.0");

NeedjaTracker.Version = ("$Revision: 1 $"):sub(12, -3) or "";

NeedjaTracker.LootMsgMatchTable = {
    { LOOT_ROLL_ALL_PASSED           , "RollAllPassed", { "item" }                  , {} },
    { LOOT_ROLL_DISENCHANT           , "RollChose"    , { "player", "item" }        , { choice = "disenchant" } },
    { LOOT_ROLL_DISENCHANT_SELF      , "RollChose"    , { "item" }                  , { player = "$self$", choice = "disenchant" } },
    { LOOT_ROLL_GREED                , "RollChose"    , { "player", "item" }        , { choice = "greed" } },
    { LOOT_ROLL_GREED_SELF           , "RollChose"    , { "item" }                  , { player = "$self$", choice = "greed" } },
    { LOOT_ROLL_NEED                 , "RollChose"    , { "player", "item" }        , { choice = "need" } },
    { LOOT_ROLL_NEED_SELF            , "RollChose"    , { "item" }                  , { player = "$self$", choice = "need" } },
    { LOOT_ROLL_PASSED               , "RollChose"    , { "player", "item" }        , { choice = "pass" } },
    { LOOT_ROLL_PASSED_AUTO          , "RollChose"    , { "player", "item" }        , { choice = "pass" } },
    { LOOT_ROLL_PASSED_AUTO_FEMALE   , "RollChose"    , { "player", "item" }        , { choice = "pass" } },
    { LOOT_ROLL_PASSED_SELF          , "RollChose"    , { "item" }                  , { player = "$self$", choice = "pass" } },
    { LOOT_ROLL_PASSED_SELF_AUTO     , "RollChose"    , { "item" }                  , { player = "$self$", choice = "pass" } },
    { LOOT_ROLL_ROLLED_DE            , "RollRolled"   , { "roll", "item", "player" }, { choice = "disenchant" } },
    { LOOT_ROLL_ROLLED_GREED         , "RollRolled"   , { "roll", "item", "player" }, { choice = "greed" } },
    { LOOT_ROLL_ROLLED_NEED          , "RollRolled"   , { "roll", "item", "player" }, { choice = "need" } },
    { LOOT_ROLL_WON                  , "RollWon"      , { "player", "item" }        , {} },
    { LOOT_ROLL_WON_NO_SPAM_DE       , "RollWonNoSpam", { "player", "roll", "item" }, { choice = "disenchant" } },
    { LOOT_ROLL_WON_NO_SPAM_GREED    , "RollWonNoSpam", { "player", "roll", "item" }, { choice = "greed" } },
    { LOOT_ROLL_WON_NO_SPAM_NEED     , "RollWonNoSpam", { "player", "roll", "item" }, { choice = "need" } },
    { LOOT_ROLL_YOU_WON              , "RollWon"      , { "item" }                  , { player = "$self$" } },
    { LOOT_ROLL_YOU_WON_NO_SPAM_DE   , "RollWonNoSpam", { "roll", "item" }          , { player = "$self$", choice = "disenchant" } },
    { LOOT_ROLL_YOU_WON_NO_SPAM_GREED, "RollWonNoSpam", { "roll", "item" }          , { player = "$self$", choice = "greed" } },
    { LOOT_ROLL_YOU_WON_NO_SPAM_NEED , "RollWonNoSpam", { "roll", "item" }          , { player = "$self$", choice = "need" } },
};

function NeedjaTracker:OnInitialize()
    local defaults = {
        char = {
            RollResults = {},
            RollStatistics = {},
            RollHistory = {},
        },
    };

    self.db = LibStub("AceDB-3.0"):New("NeedjaTrackerDB", db_defaults, true);

    self:RegisterChatCommand("needjatracker", "ProcessChatCommand");
    self:RegisterChatCommand("nt", "ProcessChatCommand");

    self:RegisterEvent("CHAT_MSG_LOOT", "ProcessChatMessage");

    self:RegisterEvent("PARTY_MEMBERS_CHANGED", "ProcessPartyEvent");
    self:RegisterEvent("RAID_ROSTER_UPDATE", "ProcessPartyEvent");
    self:RegisterEvent("PLAYER_ENTERING_WORLD", "ProcessPartyEvent");

    ChatFrame_AddMessageEventFilter("CHAT_MSG_LOOT", function(this, ...) return self:ChatMsgFilter(...) end);
end

function NeedjaTracker:ProcessChatCommand(msg)
    local args, opts = { self:GetArgs(msg, 100) }, {};
    local cmd = args[1];
    table.remove(args);
    table.remove(args, 1);
    local n, arg;
    for n, arg in pairs(args) do opts[arg] = true end;
    local channel;
    if opts.party then
        channel = "PARTY";
    elseif opts.raid then
        channel = "RAID";
    elseif opts.guild then
        channel = "GUILD";
    end;
    if cmd == "stat" or cmd == "statistics" then
        self:ShowRollStatistics(channel);
        if opts.clear then
            self:ClearRollHistory();
        end;
    elseif cmd == "hist" or cmd == "history" then
        self:ShowRollHistory(channel);
        if opts.clear then
            self:ClearRollHistory();
        end;
    elseif cmd == "clear" then
        self:ClearRollHistory();
    else
        local i;
        for i = 1, L.msg_help_total do
            PrintToChat(L["msg_help_" .. i]);
        end
    end
end

function NeedjaTracker:ProcessChatMessage(event, msg)
    local func, args = ParseMsgMatchTable(self.LootMsgMatchTable, msg);
    if func then
        self[func](self, args);
    end
end

function NeedjaTracker:ProcessPartyEvent()
    local db = self.db.char;
    local instance = IsInInstance();
    if instance then
        if db.InstanceStatus ~= instance then
            self:ClearRollHistory();
        end
        db.InstanceStatus = instance;
    end
    if IsInRaid() then
        if db.PartyStatus ~= 2 then
            if not db.PartyStatus then
                self:ClearRollHistory();
                db.RollResults = {};
            end
            db.PartyStatus = 2;
        end
    elseif IsInParty() then
        if db.PartyStatus ~= 1 then
            if not db.PartyStatus then
                self:ClearRollHistory();
                db.RollResults = {};
            end
            db.PartyStatus = 1;
        end
    else
        if db.PartyStatus then
            db.RollResults = {};
            db.PartyStatus = nil;
        end
    end
end

function NeedjaTracker:ChatMsgFilter(event, msg)
    return ParseMsgMatchTable(self.LootMsgMatchTable, msg);
end
